*-----------------------------------------------------------------------
*     TMSTLC : THE ROUTINE FOR DRAWING STREAM LINES
*-----------------------------------------------------------------------
*     COPYRIGHT (C) 2000-2012 GFD DENNOU CLUB. ALL RIGHTS RESERVED.
*-----------------------------------------------------------------------
      SUBROUTINE TMSTLC( X, Y, U, V, NX, NY )

      IMPLICIT NONE

*-- AGREENMENT
      INTEGER  NX        !* GRID NUMBERS FOR X-DIRECTION
      INTEGER  NY        !* GRID NUMBERS FOR Y-DIRECTION
      REAL     U(NX,NY)  !* VECTOR COMPONENT OF X-DIRECTION
      REAL     V(NX,NY)  !* VECTOR COMPONENT OF Y-DIRECTION
      REAL     X(NX)     !* GRID POINTS OF X-DIRECTION [UNIT:LENGTH]
      REAL     Y(NY)     !* GRID POINTS OF Y-DIRECTION [UNIT:LENGTH]

*-- INTERNAL VARIABLES
      REAL     TRX(NX*NY,NX*NY)  !* STREAM LINE OF X-COORDINATE
      REAL     TRY(NX*NY,NX*NY)  !* STREAM LINE OF Y-COORDINATE
      INTEGER  I, J              !* TMP VALUES
      INTEGER  COUNTER           !* NUMBER OF STREAM LINE
      INTEGER  NM(NX*NY)         !* ARRAY NUMBER IN EACH STREAM LINE
      REAL     ARROW_THRES
      REAL     DXA, DYA
      REAL     ARROW_LENG
      REAL     VXMIN, VXMAX, VYMIN, VYMAX
      REAL     UXMIN, UXMAX, UYMIN, UYMAX
      REAL     VXRATIO, VYRATIO
      REAL     CIRC_FLAG
      REAL     UNDEF
      INTEGER  N, M, IBLKGE, ARRCNT
      LOGICAL  NO_SHORT, END_ARR

*-- GETTING TMPACK'S PARAMETERS
      CALL TMRGET( 'ARRWINTV', ARROW_THRES )
      CALL TMLGET( 'NODRSHRT', NO_SHORT )
      CALL TMLGET( 'ENDARROW', END_ARR )
      CALL GLRGET( 'RMISS', UNDEF )
      CALL GLLSET( 'LMISS', .TRUE. )

*-- GETTING VIEWPORT PARAMETERS
      CALL SGQVPT( VXMIN, VXMAX, VYMIN, VYMAX)
      CALL SGQWND( UXMIN, UXMAX, UYMIN, UYMAX)
      VXRATIO=(VXMAX-VXMIN)/(UXMAX-UXMIN)
      VYRATIO=(VYMAX-VYMIN)/(UYMAX-UYMIN)

*-- DOING TMSTLN ROUTINE
      CALL TMSTLN( X, Y, U, V, NX, NY, TRX, TRY, NM )

*-- GETTING STREAM LINE NUMBERS
      CALL TMIGET( 'STLNNUM', COUNTER )

*-- SETTING PARAMETER FOR ARROW. (NOW, CONSTANT VALUES)
      CALL SGLSET('LPROP',.FALSE.)
      CALL SGRSET('CONST',0.01)

*-- DRAWING STREAM LINES
      DO 81 I=1,COUNTER

*-- CALCULATING THE INTERVAL LENGTH OF DRAWING ARROWS.
         ARROW_LENG=0.0
         ARRCNT=0
         IF(NM(I).GT.1)THEN
            DO 80 J=2,NM(I)
               IF(TRX(J,I).NE.UNDEF.AND.TRY(J,I).NE.UNDEF.AND.
     &            TRX(J-1,I).NE.UNDEF.AND.TRY(J-1,I).NE.UNDEF)THEN
                  DXA=(TRX(J,I)-TRX(J-1,I))*VXRATIO
                  DYA=(TRY(J,I)-TRY(J-1,I))*VYRATIO
                  ARROW_LENG=ARROW_LENG+SQRT(DXA*DXA+DYA*DYA)
                  IF(ARROW_LENG.GE.ARROW_THRES)THEN
                     CALL SGLAU( TRX(J-1,I), TRY(J-1,I),
     &                           TRX(J,I), TRY(J,I) )
                     ARROW_LENG=0.0
                     ARRCNT=ARRCNT+1
                  END IF
               END IF
 80         CONTINUE

*-- IF NO_SHORT IS TRUE, NO DRAWING STREAM LINE WHOSE ARROW NUMBER IS ZERO
*-- ήλȽˤ, ĤήȤߤʤ.
*-- 𱩤Ͻ.
            IF(ARRCNT.EQ.0)THEN
               IF(NO_SHORT.EQV..TRUE.)THEN
                  IF(NM(I).LE.2)THEN
                     GO TO 81
                  ELSE
                     N=IBLKGE( X, NX, TRX(1,I) )
                     M=IBLKGE( X, NX, TRX(NM(I),I) )
                     IF(ABS(N-M).GT.1)THEN
                        GO TO 81
                     ELSE
*-- ƤΤ, ûʤΤΥå
*--  , , νƤа.
                        CIRC_FLAG=(TRX(NM(I),I)-TRX(NM(I)-1,I))
     &                            *(TRX(1,I)-TRX(NM(I),I))
                        IF(CIRC_FLAG.LT.0.0)THEN  ! ñû
                           GO TO 81
                        ELSE
                           N=IBLKGE( Y, NY, TRY(1,I) )
                           M=IBLKGE( Y, NY, TRY(NM(I),I) )
                           IF(ABS(N-M).GT.1)THEN
                              GO TO 81
                           ELSE
                              CIRC_FLAG=(TRY(NM(I),I)-TRY(NM(I)-1,I))
     &                                  *(TRY(1,I)-TRY(NM(I),I))
                              IF(CIRC_FLAG.LT.0.0)THEN
                                 GO TO 81
!                              ELSE
!                              CALL SGLAU( TRX(NM(I)-1,I), TRY(NM(I)-1,I),
!     &                                    TRX(NM(I),I), TRY(NM(I),I) )
                              END IF
                           END IF
                        END IF
                     END IF
                  END IF
               ELSE
                  IF(END_ARR.EQV..TRUE.)THEN
                     CALL SGLAU( TRX(NM(I)-1,I), TRY(NM(I)-1,I),
     &                           TRX(NM(I),I), TRY(NM(I),I) )
                  END IF
               END IF
            END IF

            CALL UULIN( NM(I), TRX(1:NM(I),I), TRY(1:NM(I),I) )

         END IF

 81   CONTINUE

      CALL MSGDMP( 'M', 'TMSTLC', 'DRAWING FINISHED.' )

      END SUBROUTINE TMSTLC

